//
//  GsonUtil.swift
//  DemoPayme
//
//  Created by Levis Silvestre on 16/08/24.
//

import Foundation

class GsonUtil {
    
    static let instance = GsonUtil()
    
    func toJSONString(object: Any) -> String? {
        guard let dict = toDictionary(object: object) else { return nil }
        do {
            let jsonData = try JSONSerialization.data(withJSONObject: dict, options: .prettyPrinted)
            return String(data: jsonData, encoding: .utf8)
        }
        catch {
            return nil
        }
    }
    
    private func toDictionary(object: Any) -> [String: Any?]? {
        let mirror = Mirror(reflecting: object)
        var dict = [String: Any?]()
        for (key, value) in mirror.children {
            if let key = key {
                if let dictionaryOrPrimitive = toDictionaryOrPrimitive(value: value) {
                    dict[key] = dictionaryOrPrimitive
                }
                else {
                    dict[key] = NSNull()
                }
            }
        }
        return dict.isEmpty ? nil : dict
    }

    private func toDictionaryOrPrimitive(value: Any) -> Any? {
        if let optionalValue = value as? OptionalType, optionalValue.wrapped == nil {
            return nil
        }
        else if isPrimitive(value: value) {
            return value
        }
        else if let optionalValue = value as? OptionalType, let unwrapped = optionalValue.wrapped {
            return toDictionaryOrPrimitive(value: unwrapped)
        }
        else if let arrayValue = value as? [Any] {
            return arrayValue.map { toDictionaryOrPrimitive(value: $0) }
        }
        else {
            return toDictionary(object: value)
        }
    }

    private func isPrimitive(value: Any) -> Bool {
        switch value {
        case is Int, is Float, is Double, is Bool, is String:
            return true
        case let array as [Any]:
            return array.allSatisfy { isPrimitive(value: $0) }
        case let dict as [String: Any]:
            return dict.values.allSatisfy { isPrimitive(value: $0) }
        default:
            return false
        }
    }
    
}

protocol OptionalType {
    var wrapped: Any? { get }
}

extension Optional: OptionalType {
    var wrapped: Any? {
        return self.flatMap { $0 }
    }
}
