//
//  ViewController.swift
//  DemoPayme
//
//  Created by Levis Silvestre on 16/08/24.
//

import UIKit
import Payme

class ViewController: UITableViewController {

    @IBOutlet weak var inputEnviroment: UITextField!
    @IBOutlet weak var inputMerchantCode: UITextField!
    @IBOutlet weak var inputOAuthUser: UITextField!
    @IBOutlet weak var inputOAuthPassword: UITextField!
    
    @IBOutlet weak var inputOperationNumber: UITextField!
    @IBOutlet weak var inputAmount: UITextField!
    @IBOutlet weak var inputCurrency: UITextField!
    @IBOutlet weak var inputPhone: UITextField!
    @IBOutlet weak var inputEmail: UITextField!
    
    @IBOutlet weak var inputLocale: UITextField!
    @IBOutlet weak var inputPaymentMethods: UITextField!
    @IBOutlet weak var inputWalletUserCode: UITextField!
    
    var paymeRequest: PAYMEChargesRequest!
    var settingsIndentifierPicker: UIPickerView!
    var enviromentPickerView: UIPickerView!
    var envArray: [PaymeEnvironment] = [.DEVELOPMENT, .PRODUCTION]
    var enviromentSelected : PaymeEnvironment = .DEVELOPMENT
    let envOptions = [(key: 0, desc: "DEVELOPMENT"),(key: 1,desc: "PRODUCTION")]
    var events = ""
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.addDissmiskeyboard()
        self.createEnviromentPickerView()
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.setDefaultFieldsData()
    }
    
    private func setDefaultFieldsData(){
        let operationNumberStr = String((0..<6).map{ _ in "0123456789".randomElement()! })
        self.inputOperationNumber.text = "\(operationNumberStr)"
    }
    
    private func addDissmiskeyboard(){
        let tap = UITapGestureRecognizer(target: self, action: #selector(dismissableViewTapped))
        self.view.addGestureRecognizer(tap)
    }
    
    @objc func dismissableViewTapped() {
        self.view.endEditing(true)
    }
    
    @IBAction func onProcessClicked(_ sender: UIButton) {
        let customer = PAYMEAddressData(
            firstName: "Levis",
            lastName: "Silvestre",
            email: self.inputEmail.text!,
            phone: self.inputPhone.text!,
            identityDocument: PAYMEIdentityDocument(
                type: "DNI",
                identifier: "72661927",
                country: "PE"
            ),
            addressLine1: "Av casimiro Ulloa 333",
            addressLine2: nil,
            city: "Lima",
            state: "Lima",
            country: "PE"
        )
        
        let shippingBilling = PAYMEAddressData(
            firstName: "Levis",
            lastName: "Silvestre",
            email: self.inputEmail.text!,
            phone: self.inputPhone.text!,
            addressLine1: "Av casimiro Ulloa 333",
            addressLine2: nil,
            city: "Lima",
            state: "Lima",
            country: "PE"
        )
        
        var additionalFields = [String:String]()
        additionalFields["reserved1"] = "Reservado 1"
        additionalFields["reserved2"] = "Reservado 2"
        
        self.paymeRequest = PAYMEChargesRequest(
            transaction: PAYMETransactionData(
                operationNumber: self.inputOperationNumber.text!,
                amount: self.inputAmount.text!,
                currency: self.inputCurrency.text!,
                additionalFields: additionalFields,
                customer: customer,
                billing: shippingBilling,
                shipping: shippingBilling
            ),
            setting: PAYMESettingData(
                locale: self.inputLocale.text!,
                walletUserID: self.inputWalletUserCode.text!,
                paymentMethods: self.inputPaymentMethods.text!.components(separatedBy: ",")
            )
        )
        
        let paymeClient = PaymeClient(delegate: self, merchantCode: self.inputMerchantCode.text!)
        paymeClient.setEnvironment(environment: enviromentSelected)
        paymeClient.invokeCaptureForm(from: self.navigationController!, sessionToken: "SESSION_TOKEN", paymeRequest: paymeRequest)
    }

}

extension ViewController: PaymeClientDelegate {
    
    func onRespondsPayme(response: PAYMEChargesResponse) {
        let jsonRequest = GsonUtil.instance.toJSONString(object: self.paymeRequest!)
        let jsonResponse = GsonUtil.instance.toJSONString(object: response)
        
        let controller = self.storyboard?.instantiateViewController(withIdentifier: "ResponseController") as! ResponseController
        controller.paymeRequest = jsonRequest
        controller.paymeResponse = jsonResponse
        controller.paymeEvents = self.events
        self.navigationController?.pushViewController(controller, animated: true)
    }
    
    func onPaymeEvents(event: PaymeEvent) {
        var eventTXT = "Event: \(event.code.value)"
        if let event = event as? PaymeEvent.BUILD_FORM {
            eventTXT += "\nMethods: \(event.methods)"
        }
        if let event = event as? PaymeEvent.CHOOSE_PAYMENT_METHOD {
            eventTXT += "\nMethod: \(event.method)"
        }
        if let event = event as? PaymeEvent.PRESS_PAY_BUTTON {
            eventTXT += "\nMethod: \(event.method)"
        }
        if let event = event as? PaymeEvent.CANCEL_PAYMENT_METHOD {
            eventTXT += "\nMethod: \(event.method)"
        }
        self.events = "\(self.events)\n\n\(eventTXT)".trimmingCharacters(in: .whitespacesAndNewlines)
    }
    
}

extension ViewController: UIPickerViewDelegate, UIPickerViewDataSource {
    
    @objc func makeDoneInEnviromentPicker() {
        let row = enviromentPickerView.selectedRow(inComponent: 0)
        inputEnviroment.text = self.envOptions[row].desc
        inputEnviroment.resignFirstResponder()
    }
    
    func createEnviromentPickerView() {
        enviromentPickerView = UIPickerView()
        enviromentPickerView.delegate = self
        enviromentPickerView.dataSource = self
        inputEnviroment.inputAccessoryView = createToolBar(actionButton: #selector(self.makeDoneInEnviromentPicker))
        inputEnviroment.inputView = enviromentPickerView
    }
    
    func createToolBar(actionButton: Selector) -> UIToolbar {
        let toolBar = UIToolbar()
        toolBar.barStyle = UIBarStyle.default
        toolBar.isTranslucent = false
        toolBar.sizeToFit()
        let doneButton = UIBarButtonItem(title: "Hecho", style: UIBarButtonItem.Style.plain, target: self, action: actionButton)
        let spaceButton = UIBarButtonItem(barButtonSystemItem: UIBarButtonItem.SystemItem.flexibleSpace, target: nil, action: nil)
        toolBar.setItems([spaceButton, spaceButton, doneButton], animated: false)
        toolBar.isUserInteractionEnabled = true
        return toolBar
    }
    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return envOptions.count
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        self.enviromentSelected = envArray[row]
        return self.inputEnviroment.text = self.envOptions[row].desc
    }
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        return envOptions[row].desc
    }
    
}


